@@ -39,6 +39,7 @@
 #include "qgsembeddedsymbolrenderer.h"
 #include "qgsmarkersymbol.h"
 #include "qgsexpressionnodeimpl.h"
+#include "qgssldexportcontext.h"
 
 #include <QDomDocument>
 #include <QDomElement>
@@ -133,6 +134,18 @@ void QgsRendererCategory::toSld( QDomDocument &doc, QDomElement &element, QVaria
 
   QString attrName = props[ QStringLiteral( "attribute" )].toString();
 
+  QgsSldExportContext context;
+  context.setExtraProperties( props );
+  toSld( doc, element, attrName, context );
+}
+
+bool QgsRendererCategory::toSld( QDomDocument &doc, QDomElement &element, const QString &classAttribute, QgsSldExportContext &context ) const
+{
+  if ( !mSymbol.get() || classAttribute.isEmpty() )
+    return false;
+
+  QString attrName = classAttribute;
+
   // try to determine if attribute name is actually a field reference or expression.
   // If it's a field reference, we need to quote it.
   // Because we don't have access to the layer or fields here, we treat a parser error
@@ -192,20 +205,24 @@ void QgsRendererCategory::toSld( QDomDocument &doc, QDomElement &element, QVaria
     filterFunc = QStringLiteral( "%1 = %2" ).arg( attrName, QgsExpression::quotedValue( mValue ) );
   }
 
-  QgsSymbolLayerUtils::createFunctionElement( doc, ruleElem, filterFunc );
+  QgsSymbolLayerUtils::createFunctionElement( doc, ruleElem, filterFunc, context );
 
   // add the mix/max scale denoms if we got any from the callers
+  const QVariantMap oldProps = context.extraProperties();
+  QVariantMap props = oldProps;
   QgsSymbolLayerUtils::applyScaleDependency( doc, ruleElem, props );
-
-  mSymbol->toSld( doc, ruleElem, props );
+  context.setExtraProperties( props );
+  mSymbol->toSld( doc, ruleElem, context );
+  context.setExtraProperties( oldProps );
   if ( !QgsSymbolLayerUtils::hasSldSymbolizer( ruleElem ) )
   {
     // symbol could not be converted to SLD, or is an "empty" symbol. In this case we do not generate a rule, as
     // SLD spec requires a Symbolizer element to be present
-    return;
+    return false;
   }
 
   element.appendChild( ruleElem );
+  return true;
 }
 
 ///////////////////
@@ -460,7 +477,7 @@ bool labelLessThan( const QgsRendererCategory &c1, const QgsRendererCategory &c2
 
 bool labelGreaterThan( const QgsRendererCategory &c1, const QgsRendererCategory &c2 )
 {
-  return !labelLessThan( c1, c2 );
+  return QString::localeAwareCompare( c1.label(), c2.label() ) > 0;
 }
 
 void QgsCategorizedSymbolRenderer::sortByLabel( Qt::SortOrder order )
@@ -573,14 +590,27 @@ QgsCategorizedSymbolRenderer *QgsCategorizedSymbolRenderer::clone() const
 
 void QgsCategorizedSymbolRenderer::toSld( QDomDocument &doc, QDomElement &element, const QVariantMap &props ) const
 {
-  QVariantMap newProps = props;
+  QgsSldExportContext context;
+  context.setExtraProperties( props );
+  toSld( doc, element, context );
+}
+
+bool QgsCategorizedSymbolRenderer::toSld( QDomDocument &doc, QDomElement &element, QgsSldExportContext &context ) const
+{
+  const QVariantMap oldProps = context.extraProperties();
+  QVariantMap newProps = oldProps;
   newProps[ QStringLiteral( "attribute" )] = mAttrName;
+  context.setExtraProperties( newProps );
 
   // create a Rule for each range
+  bool result = true;
   for ( QgsCategoryList::const_iterator it = mCategories.constBegin(); it != mCategories.constEnd(); ++it )
   {
-    it->toSld( doc, element, newProps );
+    if ( !it->toSld( doc, element, mAttrName, context ) )
+      result = false;
   }
+  context.setExtraProperties( oldProps );
+  return result;
 }
 
 QString QgsCategorizedSymbolRenderer::filter( const QgsFields &fields )
@@ -773,6 +803,7 @@ QgsFeatureRenderer *QgsCategorizedSymbolRenderer::create( QDomElement &element,
 
   QDomElement catElem = catsElem.firstChildElement();
   int i = 0;
+  QSet<QString> usedUuids;
   while ( !catElem.isNull() )
   {
     if ( catElem.tagName() == QLatin1String( "category" ) )
@@ -801,10 +832,15 @@ QgsFeatureRenderer *QgsCategorizedSymbolRenderer::create( QDomElement &element,
       QString label = catElem.attribute( QStringLiteral( "label" ) );
       bool render = catElem.attribute( QStringLiteral( "render" ) ) != QLatin1String( "false" );
       QString uuid = catElem.attribute( QStringLiteral( "uuid" ), QString::number( i++ ) );
+      while ( usedUuids.contains( uuid ) )
+      {
+        uuid = QUuid::createUuid().toString();
+      }
       if ( symbolMap.contains( symbolName ) )
       {
         QgsSymbol *symbol = symbolMap.take( symbolName );
         cats.append( QgsRendererCategory( value, symbol, label, render, uuid ) );
+        usedUuids << uuid;
       }
     }
     catElem = catElem.nextSiblingElement();
@@ -833,7 +869,7 @@ QgsFeatureRenderer *QgsCategorizedSymbolRenderer::create( QDomElement &element,
   QDomElement sourceColorRampElem = element.firstChildElement( QStringLiteral( "colorramp" ) );
   if ( !sourceColorRampElem.isNull() && sourceColorRampElem.attribute( QStringLiteral( "name" ) ) == QLatin1String( "[source]" ) )
   {
-    r->setSourceColorRamp( QgsSymbolLayerUtils::loadColorRamp( sourceColorRampElem ) );
+    r->setSourceColorRamp( QgsSymbolLayerUtils::loadColorRamp( sourceColorRampElem ).release() );
   }
 
   QDomElement rotationElem = element.firstChildElement( QStringLiteral( "rotation" ) );
@@ -1188,12 +1224,11 @@ QSet<QString> QgsCategorizedSymbolRenderer::legendKeysForFeature( const QgsFeatu
     }
     else
     {
-      // Numeric NULL cat value is stored as an empty string
-      if ( QgsVariantUtils::isNull( value ) && ( value.userType() == QMetaType::Type::Double || value.userType() == QMetaType::Type::Int ||
-           value.userType() == QMetaType::Type::UInt || value.userType() == QMetaType::Type::LongLong ||
-           value.userType() == QMetaType::Type::ULongLong || value.userType() == QMetaType::Type::Bool ) )
+      // NULL cat value may be stored as an empty string or an invalid variant, depending on how
+      // the renderer was constructed and which QGIS version was used
+      if ( QgsVariantUtils::isNull( value ) )
       {
-        match = cat.value().toString().isEmpty();
+        match = cat.value().toString().isEmpty() || QgsVariantUtils::isNull( cat.value() );
       }
       else
       {
@@ -1445,7 +1480,10 @@ QgsCategorizedSymbolRenderer *QgsCategorizedSymbolRenderer::convertFromRenderer(
     QgsSymbolList symbols = const_cast<QgsFeatureRenderer *>( renderer )->symbols( context );
     if ( !symbols.isEmpty() )
     {
-      r->setSourceSymbol( symbols.at( 0 )->clone() );
+      QgsSymbol *newSymbol = symbols.at( 0 )->clone();
+      QgsSymbolLayerUtils::resetSymbolLayerIds( newSymbol );
+      QgsSymbolLayerUtils::clearSymbolLayerMasks( newSymbol );
+      r->setSourceSymbol( newSymbol );
     }
   }
 
@@ -1539,6 +1577,7 @@ QgsCategoryList QgsCategorizedSymbolRenderer::createCategories( const QList<QVar
     for ( const QVariant &value : vals )
     {
       QgsSymbol *newSymbol = symbol->clone();
+      QgsSymbolLayerUtils::resetSymbolLayerIds( newSymbol );
       if ( !QgsVariantUtils::isNull( value ) )
       {
         const int fieldIdx = fields.lookupField( attributeName );
@@ -1557,6 +1596,7 @@ QgsCategoryList QgsCategorizedSymbolRenderer::createCategories( const QList<QVar
 
   // add null (default) value
   QgsSymbol *newSymbol = symbol->clone();
+  QgsSymbolLayerUtils::resetSymbolLayerIds( newSymbol );
   cats.append( QgsRendererCategory( QVariant(), newSymbol, QString(), true ) );
 
   return cats;
