@@ -65,6 +65,7 @@ QgsAttributeTypeDialog::QgsAttributeTypeDialog( QgsVectorLayer *vl, int fieldIdx
   if ( vl->fields().fieldOrigin( fieldIdx ) == Qgis::FieldOrigin::Join || vl->fields().fieldOrigin( fieldIdx ) == Qgis::FieldOrigin::Expression || vl->fields().field( fieldIdx ).isReadOnly() )
   {
     isFieldEditableCheckBox->setEnabled( false );
+    mEditableExpressionButton->setEnabled( false );
   }
 
   mExpressionWidget->registerExpressionContextGenerator( this );
@@ -126,6 +127,34 @@ QgsAttributeTypeDialog::QgsAttributeTypeDialog( QgsVectorLayer *vl, int fieldIdx
   mDuplicatePolicyComboBox->addItem( tr( "Remove Value" ), QVariant::fromValue( Qgis::FieldDuplicatePolicy::UnsetField ) );
   connect( mDuplicatePolicyComboBox, qOverload<int>( &QComboBox::currentIndexChanged ), this, &QgsAttributeTypeDialog::updateDuplicatePolicyLabel );
   updateDuplicatePolicyLabel();
+
+  if ( mLayer->isSpatial() )
+  {
+    mMergePolicyComboBox->addItem( tr( "Remove Value" ), QVariant::fromValue( Qgis::FieldDomainMergePolicy::UnsetField ) );
+    mMergePolicyComboBox->addItem( tr( "Use Default Value" ), QVariant::fromValue( Qgis::FieldDomainMergePolicy::DefaultValue ) );
+
+    if ( mLayer->fields().at( mFieldIdx ).isNumeric() )
+    {
+      mMergePolicyComboBox->addItem( tr( "Use Sum" ), QVariant::fromValue( Qgis::FieldDomainMergePolicy::Sum ) );
+      mMergePolicyComboBox->addItem( tr( "Use Maximum Value" ), QVariant::fromValue( Qgis::FieldDomainMergePolicy::MaximumValue ) );
+      mMergePolicyComboBox->addItem( tr( "Use Minimum Value" ), QVariant::fromValue( Qgis::FieldDomainMergePolicy::MinimumValue ) );
+
+      if ( mLayer->geometryType() != Qgis::GeometryType::Point )
+        mMergePolicyComboBox->addItem( tr( "Use Average Weighted by Geometry" ), QVariant::fromValue( Qgis::FieldDomainMergePolicy::GeometryWeighted ) );
+    }
+
+    mMergePolicyComboBox->addItem( tr( "Use Largest Feature" ), QVariant::fromValue( Qgis::FieldDomainMergePolicy::LargestGeometry ) );
+    mMergePolicyComboBox->addItem( tr( "Set to NULL" ), QVariant::fromValue( Qgis::FieldDomainMergePolicy::SetToNull ) );
+  }
+  else
+  {
+    mMergePolicyComboBox->setEnabled( false );
+    mMergePolicyLabel->setEnabled( false );
+    mMergePolicyDescriptionLabel->hide();
+  }
+
+  connect( mMergePolicyComboBox, qOverload<int>( &QComboBox::currentIndexChanged ), this, &QgsAttributeTypeDialog::updateMergePolicyLabel );
+  updateMergePolicyLabel();
 }
 
 QgsAttributeTypeDialog::~QgsAttributeTypeDialog()
@@ -208,6 +237,21 @@ void QgsAttributeTypeDialog::setEditorWidgetType( const QString &type, bool forc
     }
   }
 
+  // "Editable" and "Reuse last entered value" checkboxes should be disabled for read-only widgets like JsonEdit
+  // see https://github.com/qgis/QGIS/issues/47755
+  if ( QgsGui::editorWidgetRegistry()->isReadOnly( type ) )
+  {
+    isFieldEditableCheckBox->setEnabled( false );
+    mEditableExpressionButton->setEnabled( false );
+    reuseLastValuesCheckBox->setEnabled( false );
+  }
+  else
+  {
+    isFieldEditableCheckBox->setEnabled( true );
+    mEditableExpressionButton->setEnabled( true );
+    reuseLastValuesCheckBox->setEnabled( true );
+  }
+
   //update default expression preview
   defaultExpressionChanged();
 }
@@ -344,7 +388,6 @@ int QgsAttributeTypeDialog::fieldIdx() const
   return mFieldIdx;
 }
 
-
 QgsExpressionContext QgsAttributeTypeDialog::createExpressionContext() const
 {
   QgsExpressionContext context;
@@ -400,6 +443,17 @@ void QgsAttributeTypeDialog::setDuplicatePolicy( Qgis::FieldDuplicatePolicy poli
   updateSplitPolicyLabel();
 }
 
+void QgsAttributeTypeDialog::setMergePolicy( Qgis::FieldDomainMergePolicy policy )
+{
+  mMergePolicyComboBox->setCurrentIndex( mMergePolicyComboBox->findData( QVariant::fromValue( policy ) ) );
+  updateMergePolicyLabel();
+}
+
+Qgis::FieldDomainMergePolicy QgsAttributeTypeDialog::mergePolicy() const
+{
+  return mMergePolicyComboBox->currentData().value<Qgis::FieldDomainMergePolicy>();
+}
+
 QString QgsAttributeTypeDialog::constraintExpression() const
 {
   return constraintExpressionWidget->asExpression();
@@ -537,6 +591,46 @@ void QgsAttributeTypeDialog::updateDuplicatePolicyLabel()
   mDuplicatePolicyDescriptionLabel->setText( QStringLiteral( "<i>%1</i>" ).arg( helperText ) );
 }
 
+void QgsAttributeTypeDialog::updateMergePolicyLabel()
+{
+  QString helperText;
+  switch ( mMergePolicyComboBox->currentData().value<Qgis::FieldDomainMergePolicy>() )
+  {
+    case Qgis::FieldDomainMergePolicy::DefaultValue:
+      helperText = tr( "Use default field value." );
+      break;
+
+    case Qgis::FieldDomainMergePolicy::Sum:
+      helperText = tr( "Sum of values." );
+      break;
+
+    case Qgis::FieldDomainMergePolicy::GeometryWeighted:
+      helperText = tr( "New values are computed as the weighted average of the source values." );
+      break;
+
+    case Qgis::FieldDomainMergePolicy::UnsetField:
+      helperText = tr( "Clears the field to an unset state." );
+      break;
+
+    case Qgis::FieldDomainMergePolicy::LargestGeometry:
+      helperText = tr( "Use value from feature with the largest geometry." );
+      break;
+
+    case Qgis::FieldDomainMergePolicy::MinimumValue:
+      helperText = tr( "Use the lowest value from the selected features." );
+      break;
+
+    case Qgis::FieldDomainMergePolicy::MaximumValue:
+      helperText = tr( "Use the highest value from the selected features." );
+      break;
+
+    case Qgis::FieldDomainMergePolicy::SetToNull:
+      helperText = tr( "Set attribute to NULL." );
+      break;
+  }
+  mMergePolicyDescriptionLabel->setText( QStringLiteral( "<i>%1</i>" ).arg( helperText ) );
+}
+
 QStandardItem *QgsAttributeTypeDialog::currentItem() const
 {
   QStandardItemModel *widgetTypeModel = qobject_cast<QStandardItemModel *>( mWidgetTypeComboBox->model() );
